@@ -20,25 +20,23 @@
  */
 
 #include "timelinetools.h"
+#include "kio_timeline_debug.h"
 
 #include <QUrl>
-#include <KCalendarSystem>
-#include <KGlobal>
-#include <KLocalizedString>
-#include <QDebug>
+#include <QDate>
+#include <QRegularExpression>
+#include <QUrlQuery>
 
-#include <QtCore/QDate>
-#include <QtCore/QRegExp>
 
 namespace
 {
 QDate applyRelativeDateModificators(const QDate& date, const QMap<QString, QString>& modificators)
 {
     QDate newDate(date);
-    const QString dayKey = QLatin1String("relDays");
-    const QString weekKey = QLatin1String("relWeeks");
-    const QString monthKey = QLatin1String("relMonths");
-    const QString yearKey = QLatin1String("relYears");
+    const QString dayKey = QStringLiteral("relDays");
+    const QString weekKey = QStringLiteral("relWeeks");
+    const QString monthKey = QStringLiteral("relMonths");
+    const QString yearKey = QStringLiteral("relYears");
     bool ok = false;
 
     if (modificators.contains(yearKey)) {
@@ -56,8 +54,7 @@ QDate applyRelativeDateModificators(const QDate& date, const QMap<QString, QStri
     if (modificators.contains(weekKey)) {
         int relWeeks = modificators[weekKey].toInt(&ok);
         if (ok) {
-            const KCalendarSystem* calSystem = KGlobal::locale()->calendar();
-            newDate = newDate.addDays(relWeeks * calSystem->daysInWeek(date));
+            newDate = newDate.addDays(relWeeks * 7); // we assume weeks have 7 days everywhere. QDate seems to make that assumption too, should be OK.
         }
     }
     if (modificators.contains(dayKey)) {
@@ -70,65 +67,88 @@ QDate applyRelativeDateModificators(const QDate& date, const QMap<QString, QStri
 }
 }
 
+QUrl Baloo::canonicalizeTimelineUrl(const QUrl& url) {
+    QUrl newUrl = url;
+    QString path = url.path();
+    if (path.contains(QLatin1String("//"))) {
+        QStringList sections = path.split(QChar('/'), QString::SkipEmptyParts);
+        path = '/' + sections.join(QChar('/'));
+        newUrl.setPath(path);
+    }
+    if ((path.size() > 1) && path.endsWith(QChar('/'))) {
+        path.chop(1);
+        newUrl.setPath(path);
+    }
+    if (!path.startsWith(QChar('/'))) {
+        path = '/' + path;
+        newUrl.setPath(path);
+    }
+    return newUrl;
+}
 
 Baloo::TimelineFolderType Baloo::parseTimelineUrl(const QUrl& url, QDate* date, QString* filename)
 {
-    qDebug() << url;
+    qCDebug(KIO_TIMELINE) << url;
 
-    static QRegExp s_dateRegexp(QLatin1String("\\d{4}-\\d{2}(?:-(\\d{2}))?"));
+    static const QRegularExpression s_dateRegexp(
+                    QRegularExpression::anchoredPattern(QStringLiteral("\\d{4}-\\d{2}(?:-(\\d{2}))?")));
 
     // reset
     *date = QDate();
 
     QString path = url.path();
-    if (path.endsWith('/'))
-        path = path.mid(0, path.length()-1);
+    if (path.endsWith(QLatin1Char('/')))
+        path.chop(1);
 
-    if (path.isEmpty() || path == QLatin1String("/")) {
-        qDebug() << url << "is root folder";
+    if (path.isEmpty()) {
+        qCDebug(KIO_TIMELINE) << url << "is root folder";
         return RootFolder;
     } else if (path.startsWith(QLatin1String("/today"))) {
         *date = QDate::currentDate();
         if (filename)
             *filename = path.mid(7);
-        qDebug() << url << "is today folder:" << *date;
+        qCDebug(KIO_TIMELINE) << url << "is today folder:" << *date;
         return DayFolder;
     } else if (path == QLatin1String("/calendar")) {
-        qDebug() << url << "is calendar folder";
+        qCDebug(KIO_TIMELINE) << url << "is calendar folder";
         return CalendarFolder;
     } else {
-        QStringList sections = path.split(QLatin1String("/"), QString::SkipEmptyParts);
+        QStringList sections = path.split(QStringLiteral("/"), QString::SkipEmptyParts);
         QString dateString;
-        if (s_dateRegexp.exactMatch(sections.last())) {
+        QRegularExpressionMatch match = s_dateRegexp.match(sections.last());
+        if (match.hasMatch()) {
             dateString = sections.last();
-        } else if (sections.count() > 1 && s_dateRegexp.exactMatch(sections[sections.count() - 2])) {
+        } else if (sections.count() > 1
+                   && (match = s_dateRegexp.match(sections[sections.count() - 2])).hasMatch()) {
             dateString = sections[sections.count() - 2];
-            if (filename)
+            if (filename) {
                 *filename = sections.last();
+            }
         } else {
-            qDebug() << url << "COULD NOT PARSE";
+            qCWarning(KIO_TIMELINE) << url << "COULD NOT PARSE";
             return NoFolder;
         }
 
-        if (s_dateRegexp.cap(1).isEmpty()) {
+        if (match.captured(1).isEmpty()) {
             // no day -> month listing
-            qDebug() << "parsing " << dateString;
-            *date = QDate::fromString(dateString, QLatin1String("yyyy-MM"));
-            qDebug() << url << "is month folder:" << date->month() << date->year();
+            qCDebug(KIO_TIMELINE) << "parsing " << dateString;
+            *date = QDate::fromString(dateString, QStringLiteral("yyyy-MM"));
+            qCDebug(KIO_TIMELINE) << url << "is month folder:" << date->month() << date->year();
             if (date->month() > 0 && date->year() > 0)
                 return MonthFolder;
         } else {
-            qDebug() << "parsing " << dateString;
+            qCDebug(KIO_TIMELINE) << "parsing " << dateString;
             typedef QPair<QString, QString> StringPair;
-            QList<StringPair> queryItems = url.queryItems();
+            QUrlQuery query(url);
+            const QList<StringPair> queryItems = query.queryItems();
             QMap<QString, QString> map;
-            Q_FOREACH (const StringPair& pair, queryItems) {
+            for (const StringPair& pair : queryItems) {
                 map.insert(pair.first, pair.second);
             }
 
-            *date = applyRelativeDateModificators(QDate::fromString(dateString, "yyyy-MM-dd"), map);
+            *date = applyRelativeDateModificators(QDate::fromString(dateString, QStringLiteral("yyyy-MM-dd")), map);
             // only in day folders we can have filenames
-            qDebug() << url << "is day folder:" << *date;
+            qCDebug(KIO_TIMELINE) << url << "is day folder:" << *date;
             if (date->isValid())
                 return DayFolder;
         }
